//MCbx 2014, GPL
#include "modelselector.h"
#include "ui_modelselector.h"

#include <QGraphicsView>
#include <QGraphicsScene>
#include <QScrollBar>
#include "../testsheet.h"
#include "icvisualizer.h"
#include <QSettings>
#include <QList>

ModelSelector::ModelSelector(QWidget *parent, QString initPath, QString settingsPath) :
    QDialog(parent),
    ui(new Ui::ModelSelector)
{

    //restore settings from ini file
    // - window size
    // - last used directory
    // - splitter sizes
    QString p = settingsPath;
    this->iniPath=p;
    QSettings settings(this->iniPath,QSettings::IniFormat);
    settings.beginGroup("OpenDialog");
    int ww = settings.value("Width","720").toInt();
    int wh = settings.value("Height","406").toInt();
    int sh1= settings.value("TreeWidth","160").toInt();
    int sh2= settings.value("FilesWidth","400").toInt();
    int sh3= settings.value("PreviewWidth","160").toInt();
    if (initPath=="") initPath=settings.value("LastPlace",initPath=QDir::currentPath()).toString();
    settings.endGroup();

    //modify path
    this->fileName="";
    ui->setupUi(this);
    dirmodel = new QFileSystemModel(this);
    this->setWindowTitle("Select test sheet");
    this->directory=initPath;
    if ((initPath=="")||(!QDir(initPath).exists())) this->directory=QDir::currentPath();

    //prepare tree for displaying only directories
    dirmodel->setFilter(QDir::AllDirs|QDir::NoDotAndDotDot);
    dirmodel->setRootPath(this->directory);
    this->ui->dirView->setModel(dirmodel);
    this->ui->dirView->hideColumn(2);
    this->ui->dirView->hideColumn(1);
    this->ui->dirView->hideColumn(3);

    //open tree view to show directory
    QModelIndex qmi= dirmodel->index(this->directory);
    while(qmi.isValid())
    {
        this->ui->dirView->expand(qmi);
        qmi = qmi.parent();
    }
    ui->dirView->resizeColumnToContents(0);
    ui->dirView->header()->setResizeMode(0,QHeaderView::ResizeToContents);
    ui->dirView->setColumnWidth(0,600);
    ui->dirView->header()->close();

    //Prepare file list widget
    QStringList aaa;
    aaa<<"File"<<"Type/description"<<"Path";
    ui->twFiles->setHeaderLabels(aaa);
    ui->twFiles->setColumnHidden(2,1);
    ui->twFiles->setRootIsDecorated(0);
    ui->twFiles->setEditTriggers(QAbstractItemView::NoEditTriggers);

    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(0);

    this->previewVisualizer = new ICVisualizer(this,ui->preview,0,"");

    //apply settings from ini
    ModelSelector::resize(ww,wh);
    QList<int> sizes;
    sizes<<sh1<<sh2<<sh3;
    ui->splitter->setSizes(sizes);
    this->on_dirView_clicked(dirmodel->index(this->directory));   //show contents in file manager

}

ModelSelector::~ModelSelector()
{
    delete ui;
}

void ModelSelector::on_dirView_clicked(const QModelIndex &index)
{
    ui->twFiles->clear();
        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(0);

    this->directory = dirmodel->fileInfo(index).absoluteFilePath();
    //we've the path, we need to list mod files, verify them and show list to user.

   QDir fileLister(this->directory,"*.mod");
    QStringList models = fileLister.entryList();

    ui->twFiles->setSortingEnabled(0);
    foreach (QString a, models)
    {
        QFileInfo tmp(this->directory,a);
        QTreeWidgetItem *entry=new QTreeWidgetItem();
        entry->setText(0,a);

        //Now let's read description from file. VERY bad thing here.
        QString desc="";
        QFile plik(tmp.absoluteFilePath());
        if (plik.open(QIODevice::ReadOnly|QIODevice::Text))
        {
            QTextStream in(&plik);
            QString line=in.readLine();
            if ((line.length()>0)&&(line.length()<17)) desc+=line.trimmed()+": ";
            line=in.readLine();
            if ((line.length()>0)&&(line.length()<49)) desc+=line.trimmed();
        }
        else desc="[CANNOT READ DESCRIPTION]";
        entry->setText(1,desc);

        entry->setText(2,tmp.absoluteFilePath());

        //This is a whole search logic
        if ((ui->edFilter->text()=="")||(desc.toLower().contains(ui->edFilter->text().toLower()))||(a.toLower().contains(ui->edFilter->text().toLower())))
            ui->twFiles->addTopLevelItem(entry);
    }
    ui->twFiles->resizeColumnToContents(1);
    ui->twFiles->sortByColumn(0,Qt::AscendingOrder);
    ui->twFiles->setSortingEnabled(1);

    ui->dirView->scrollTo(index);
    QRect r = ui->dirView->visualRect(index);
    ui->dirView->horizontalScrollBar()->setValue(r.x()-20);
}

//search causes refresh, refresh does everything
void ModelSelector::on_edFilter_textChanged()
{
    this->on_dirView_clicked(dirmodel->index(this->directory));
    this->previewVisualizer->setNumOfPins(0);
}

void ModelSelector::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);
    if (stdButton==QDialogButtonBox::Ok)
    {
        //send file name
        this->fileName=ui->twFiles->selectedItems()[0]->text(2);
        this->saveIni();
    }
    else
    {
        this->fileName="";
    }
}

void ModelSelector::on_twFiles_clicked()
{
    if (ui->twFiles->selectedItems().count()==0)
    {
        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(0);
        this->previewVisualizer->setNumOfPins(0);
    }
    else
    {
             ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(1);
             TestSheet t(ui->twFiles->selectedItems()[0]->text(2));

             this->previewVisualizer->setNumOfPins(t.getNumOfPins());
             this->previewVisualizer->setIO(t.initIO(1));
             this->previewVisualizer->setPinDescriptions(t.getPinsDescriptions());
             this->previewVisualizer->setSummary(t.getName(),3);

    }
}

QString ModelSelector::getFileName()
{
    return this->fileName;
}

void ModelSelector::on_dirView_expanded(const QModelIndex &index)
{
    ui->dirView->scrollTo(index);
    QRect r = ui->dirView->visualRect(index);
    ui->dirView->horizontalScrollBar()->setValue(r.x()-20);

}

void ModelSelector::on_twFiles_doubleClicked()
{
    if (ui->twFiles->selectedItems().count()>0)
    {
        //send file name
        this->fileName=ui->twFiles->selectedItems()[0]->text(2);
        this->saveIni();
        this->accept();
    }
}

void ModelSelector::saveIni()
{
    // write ini.
    QSettings settings(this->iniPath,QSettings::IniFormat);
    settings.beginGroup("OpenDialog");
    int tmp=ModelSelector::size().width();
    settings.setValue("Width",QString::number(tmp));
    tmp=ModelSelector::size().height();
    settings.setValue("Height",QString::number(tmp));
    settings.setValue("TreeWidth",QString::number(ui->splitter->sizes()[0]));
    settings.setValue("FilesWidth",QString::number(ui->splitter->sizes()[1]));
    settings.setValue("PreviewWidth",QString::number(ui->splitter->sizes()[2]));

    settings.setValue("LastPlace", this->directory);
    settings.endGroup();
}
